﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using Microsoft.Practices.Composite.Presentation.Commands;
using Infrastructure.Desktop;
using Microsoft.Practices.Composite.Events;
using System.ComponentModel;

namespace Module.People
{
    public class PeoplePresentationModel : IPeoplePresentationModel, INotifyPropertyChanged
    {
        #region IPeoplePresentationModel Members

        public IPeopleView View { get; private set; }

        #endregion

        #region Properties

        private ObservableCollection<object> _people;
        public ObservableCollection<object> People 
        {
            get { return _people; }
            private set
            {
                if (_people != value)
                {
                    _people = value;
                    PropertyChanged(this, new PropertyChangedEventArgs("People"));
                }
            }
        }
        public string SelectedPerson { get; private set; }

        #endregion Properties

        public DelegateCommand<object> PopulatePeopleCommand;
        public DelegateCommand<object> DeleteCommand;

        private GlobalCommandsProxy _commandProxy;
        private IEventAggregator _eventAggregator;

        public PeoplePresentationModel(IPeopleView view, GlobalCommandsProxy commandProxy, IEventAggregator eventAggregator)
        {
            //assign the view, the view's model, and hook into it's selection changed event so we can perform logic
            //when a person has been selected
            this.View = view;
            this.View.Model = this;
            this.View.OnPersonSelected += new EventHandler<System.Windows.Controls.SelectionChangedEventArgs>(View_OnPersonSelected);

            this._commandProxy = commandProxy;
            this._eventAggregator = eventAggregator;

            //lets set the execute and can execute methods for our delete command
            DeleteCommand = new DelegateCommand<object>(Delete, CanDelete);
            //now we have to hook into the delete button in the toolbar module. we do this by using the command proxy class
            //and registering our delete command with the command proxy's delete command
            commandProxy.DeleteCommand.RegisterCommand(DeleteCommand);

            //lets set the execute and can execute methods for our populate people command
            PopulatePeopleCommand = new DelegateCommand<object>(PopulatePeople, CanPopulatePeople);
            //hook into the populate people button
            commandProxy.PopulatePeopleCommand.RegisterCommand(PopulatePeopleCommand);
        }

        #region Commands

        public void PopulatePeople(object parameter)
        {
            //lets create some people
            People = new ObservableCollection<object> { "Brian Lagunas", "David Starr", "Scott Nichols" };

            //after we add our people lets reasses the state of the populate people button
            PopulatePeopleCommand.RaiseCanExecuteChanged();
        }

        public bool CanPopulatePeople(object parameter)
        {
            bool result = false;
            
            //if there are no people we can populate the list with people
            if (People == null || People.Count == 0)
            {
                result = true;
            }

            return result;
        }

        public void Delete(object parameter)
        {
            //remove the selected peron
            People.Remove(this.SelectedPerson);

            //now that we have delete a person lets notify any modue that may be listening
            _eventAggregator.GetEvent<PersonDeletedEvent>().Publish(this.SelectedPerson);

            //set the selected person to null
            this.SelectedPerson = null;

            //raise the can execute event to change the state of the delete button and the populate people button
            DeleteCommand.RaiseCanExecuteChanged();
            PopulatePeopleCommand.RaiseCanExecuteChanged();
        }

        public bool CanDelete(object parameter)
        {
            bool result = false;

            //if the selected person is null or an empty string the cannot delete, otherwise, lets delete
            if (SelectedPerson != null && SelectedPerson.Length > 0)
            {
                result = true;
            }

            return result;
        }

        #endregion Commands

        #region Events

        void View_OnPersonSelected(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            //make sure there are selected items
            if (e.AddedItems.Count > 0)
            {
                //get the selected item and assign it to the selected person property
                SelectedPerson = (string)e.AddedItems[0];

                //now we raise the can execute event to change the state of the delete button
                DeleteCommand.RaiseCanExecuteChanged();

                //now lets fire the person selection changed event and pass the name of the selected person for any module
                //that may be listening for it.
                _eventAggregator.GetEvent<PersonSelectionChangedEvent>().Publish(this.SelectedPerson);
            }
        }

        #endregion Events

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged = delegate { };

        #endregion
    }
}
